package movies

import (
	"fmt"
	"net"
	"net/http"
	"net/url"

	"github.com/odwrtw/papi"
	polochon "github.com/odwrtw/polochon/lib"
	"github.com/odwrtw/polochon/modules/pam"
	"github.com/odwrtw/trakttv"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/auth"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/config"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/users"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/web"
)

// ErrPolochonUnavailable is an error returned if the polochon server is not available
var ErrPolochonUnavailable = fmt.Errorf("Invalid polochon address")

func getPolochonMovies(user *users.User) ([]*Movie, error) {
	movies := []*Movie{}

	var polochonConfig config.UserPolochon
	err := user.GetConfig("polochon", &polochonConfig)
	if err != nil {
		return movies, err
	}

	client, err := papi.New(polochonConfig.URL)
	if err != nil {
		return movies, err
	}

	if polochonConfig.Token != "" {
		client.SetToken(polochonConfig.Token)
	}

	pmovies, err := client.GetMovies()
	if err != nil {
		// Catch network error for accessing specified polochon address
		if uerr, ok := err.(*url.Error); ok {
			if nerr, ok := uerr.Err.(*net.OpError); ok {
				if nerr.Op == "dial" {
					return movies, ErrPolochonUnavailable
				}

			}
		}
		return movies, err
	}
	for _, pmovie := range pmovies {
		movie := New(pmovie.ImdbID)
		movie.PolochonURL, err = client.DownloadURL(&papi.Movie{ImdbID: movie.ImdbID})
		if err != nil {
			return nil, err
		}
		movies = append(movies, movie)
	}
	return movies, nil
}

func FromPolochon(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	v := auth.GetCurrentUser(r, env.Log)
	user, ok := v.(*users.User)
	if !ok {
		return fmt.Errorf("invalid user type")
	}

	movies, err := getPolochonMovies(user)
	if err != nil {
		return env.RenderError(w, err)
	}

	var polochonConfig config.UserPolochon
	err = user.GetConfig("polochon", &polochonConfig)
	if err != nil {
		return err
	}

	detailer, err := pam.New(&pam.Params{
		Endpoint: polochonConfig.URL,
		Token:    polochonConfig.Token,
	})

	for _, m := range movies {
		m.Detailers = []polochon.Detailer{detailer}
		err := m.GetDetails(env, false)
		if err != nil {
			env.Log.Error(err)
		}
	}

	return env.RenderJSON(w, movies)
}

// ExplorePopular returns the popular movies
func ExplorePopular(env *web.Env, w http.ResponseWriter, r *http.Request) error {
	log := env.Log.WithField("function", "movies.ExplorePopular")

	queryOption := trakttv.QueryOption{
		ExtendedInfos: []trakttv.ExtendedInfo{
			trakttv.ExtendedInfoMin,
		},
		Pagination: trakttv.Pagination{
			Page:  1,
			Limit: 20,
		},
	}
	trakt := trakttv.New(env.Config.TraktTVClientID)
	trakt.Endpoint = trakttv.ProductionEndpoint

	log.Debug("getting movies from trakttv")
	tmovies, err := trakt.PopularMovies(queryOption)
	if err != nil {
		return err
	}
	log.Debugf("got %d movies from trakttv", len(tmovies))

	movies := []*Movie{}
	ids := []string{}
	for _, m := range tmovies {
		movie := New(m.IDs.ImDB)
		err := movie.GetDetails(env, false)
		if err != nil {
			env.Log.Error(err)
			continue
		}
		movies = append(movies, movie)
		ids = append(ids, m.IDs.ImDB)
	}

	return env.RenderJSON(w, movies)
}
